<?php

namespace Drupal\wordfilter\Plugin\Filter;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Language\LanguageInterface;
use Drupal\filter\FilterProcessResult;
use Drupal\filter\Plugin\FilterBase;

/**
 * @Filter(
 *   id = "wordfilter",
 *   title = @Translation("Apply filtering of words"),
 *   description = @Translation("Filter out words by given Wordfilter configurations (choose below)."),
 *   type = Drupal\filter\Plugin\FilterInterface::TYPE_TRANSFORM_IRREVERSIBLE,
 * )
 */
class WordFilter extends FilterBase {

  /**
   * A list of all active Wordfilter configurations.
   *
   * @see ::activeWordfilterConfigs()
   *
   * @var \Drupal\wordfilter\Entity\WordfilterConfigurationInterface[]
   */
  protected $active_configs = NULL;

  /**
   * Get a list of active Wordfilter configurations for this filter.
   *
   * @return \Drupal\wordfilter\Entity\WordfilterConfigurationInterface[]
   */
  public function activeWordfilterConfigs() {
    if (!isset($this->active_configs)) {
      $settings_active = !empty($this->settings['active_wordfilter_configs']) ?
        $this->settings['active_wordfilter_configs'] : [];

      if (!empty($settings_active)) {
        $storage = \Drupal::entityTypeManager()
          ->getStorage('wordfilter_configuration');

        $this->active_configs = $storage->loadMultiple($settings_active);
      }
      else {
        $this->active_configs = [];
      }
    }

    return $this->active_configs;
  }

  /**
   * {@inheritdoc}
   */
  public function process($text, $langcode) {
    $active_configs = $this->activeWordfilterConfigs();

    if (!empty($active_configs)) {
      foreach ($active_configs as $wordfilter_config) {
        $text = $this->filterWords($text, $wordfilter_config, $langcode);
      }
    }

    return new FilterProcessResult($text);
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $storage = \Drupal::entityTypeManager()->getStorage('wordfilter_configuration');
    $wordfilter_configs = $storage->loadMultiple();

    foreach ($wordfilter_configs as $key => $config) {
      $wordfilter_configs[$key] = $config->label();
    }

    $form['active_wordfilter_configs'] = array(
      '#type' => 'select',
      '#title' => $this->t('Active Wordfilter configurations'),
      '#options' => $wordfilter_configs,
      '#default_value' => !empty($this->settings['active_wordfilter_configs']) ?
        $this->settings['active_wordfilter_configs'] : array(),
      '#description' => $this->t('Choose the available Wordfilter configurations as active filters.'),
      '#multiple' => TRUE,
    );

    return $form;
  }

  /**
   * Provides filtering of words by given Wordfilter configuration.
   *
   * TODO Add further and more intelligent filtering algorithms.
   *      The algorithm should be selected at the Wordfilter configuration.
   *      Consider the use of an external web service.
   *
   * @param string $text
   *  The text to filter.
   * @param \Drupal\wordfilter\Entity\WordfilterConfigurationInterface $wordfilter_config
   *  The Wordfilter configuration to use during the filtering process.
   * @param string $langcode
   *  (Optional) The language code.
   *  This param has currently no impact on the
   *  filtering process itself, but may be used in future.
   *
   * @return string
   *   The filtered text.
   */
  public function filterWords($text, $wordfilter_config, $langcode = LanguageInterface::LANGCODE_NOT_SPECIFIED) {
    $filter_words = $wordfilter_config->getFilterWords();
    $filter_pattern = '/\b(' . implode('|', $filter_words) . ')\b/i';
    $substitute = $wordfilter_config->getSubstitute();

    $text = preg_replace($filter_pattern, $substitute, $text);

    return $text;
  }
}
